local _,L = ...
local rematch = Rematch
local panel = RematchTeamPanel
local settings, saved

local workingList = {} -- list of teams (keys) being displayed
local searchText = nil -- desensitized text (or petID) entered in search box
local searchTabs = {} -- lookup table of tabs that have a search match

rematch:InitModule(function()
	rematch.TeamPanel = panel
	settings = RematchSettings
	saved = RematchSaved

	local scrollFrame = panel.List.ScrollFrame
	scrollFrame.update = panel.UpdateList
	scrollFrame.ScrollBar.doNotHide = true
	scrollFrame.stepSize = 264 -- 44*6 or 6 rows
	HybridScrollFrame_CreateButtons(scrollFrame,"RematchTeamListButtonTemplate",0,0)
	for _,button in ipairs(scrollFrame.buttons) do
		button:RegisterForClicks("AnyUp")
		for i=1,3 do
			button.Pets[i]:RegisterForClicks("AnyUp")
		end
	end
	panel.Top.Team:RegisterForClicks("AnyUp")
	panel.Top.Teams:SetText(L["Teams"])
	rematch:UpdateAutoLoadState()
end)

function panel:Update()
	if panel:IsVisible() then
		panel:PopulateTeamList()
		panel:UpdateLoadedTeam()
		panel:UpdateList()
	end
end

function panel:UpdateLoadedTeam()
	panel.Top.Toggle:SetEnabled(settings.loadedTeam and true)
	if settings.ShowLoadedTeam and settings.loadedTeam then
		panel.Top:SetHeight(88)
		panel.Top.Team:Show()
		panel:FillTeamButton(panel.Top.Team,settings.loadedTeam)
	else
		panel.Top:SetHeight(29)
		panel.Top.Team:Hide()
	end
	rematch:SetTopToggleButton(panel.Top.Toggle,settings.ShowLoadedTeam)
end

function panel:ToggleLoadedTeam()
	settings.ShowLoadedTeam = not settings.ShowLoadedTeam
	panel:Update()
end

--[[ Working List ]]

-- sorts teams by name with white(npcID'ed) teams listed before gold (no npcID) ones
local function teamSort(e1,e2)
	if e1 and not e2 then
		return true
	elseif not e1 and e2 then
		return false
	end
	local title1 = rematch:GetTeamTitle(e1):lower()
	local title2 = rematch:GetTeamTitle(e2):lower()
	if title1==title2 then
		local type1 = type(e1)
		local type2 = type(e2)
		if type1=="number" and type2~="number" then
			return true
		elseif type2=="number" and type1~="number" then
			return false
		else
			return e1<e2
		end
	else
		return title1<title2
	end
end

function panel:PopulateTeamList()
	local selected = rematch.TeamTabs:GetSelectedTab()
	local numTabs = #settings.TeamGroups

--	if not settings.TeamGroups[selected] then
--		selected = 1
--	end

	local forPet = searchText and rematch:GetIDType(searchText)=="pet"

	wipe(workingList)
	wipe(searchTabs)

	for key,data in pairs(saved) do
		local tab = data.tab
		if tab and tab>numTabs then
			saved[key].tab = nil
			tab = nil
		end
		if not searchText then -- not searching for a team, fill with teams in the selected tab
			if (tab and tab==selected) or (not tab and selected==1) then
				tinsert(workingList,key)
			end
		else -- there's a search happening
			if forPet then -- this is a search for a petID BattlePet-0-000etc
				for i=1,3 do
					if data[i][1]==searchText then
						tinsert(workingList,key)
						searchTabs[data.tab or 1] = true
						break
					end
				end
			else -- this is a traditional [Ss][Ee][Aa][Rr][Cc][Hh]
				if panel:TeamMatchesSearch(key,data) then
					tinsert(workingList,key)
					searchTabs[data.tab or 1] = true
				end
			end
		end
	end

	-- alphabetically sort the teams
	table.sort(workingList,teamSort) -- function(e1,e2) return rematch:GetTeamTitle(e1):lower()<rematch:GetTeamTitle(e2):lower() end)

	local custom = settings.TeamGroups[selected][3]
	if not searchText and custom then
		-- remove any teams from custom sort that don't belong in this tab
		for i=#custom,1,-1 do
			local key = custom[i]
			local team = saved[key]
			if not team or not ((team.tab and team.tab==selected) or (not team.tab and selected==1)) then
				tremove(custom,i)
			end
		end
		-- add any in workinglist that belong in custom sort but don't already
		for i=1,#workingList do
			if not tContains(custom,workingList[i]) then
				tinsert(custom,workingList[i])
			end
		end
		-- wipe workinglist and fill with custom
		wipe(workingList)
		for i=1,#custom do
			tinsert(workingList,custom[i])
		end
	end

end

-- returns true if the key,data team matches the searchText
function panel:TeamMatchesSearch(key,data)
	-- look for match in team name
	if rematch:GetTeamTitle(key):match(searchText) then
		return true
	end
	-- look for match with npc name
	if type(key)=="number" and rematch:GetNameFromNpcID(key):match(searchText) then
		return true
	end
	-- look for match in notes
	if data.notes and data.notes:match(searchText) then
		return true
	end
	-- look for match in pet names
	for i=1,3 do
		local petID = data[i][1]
		local idType = rematch:GetIDType(petID)
		if idType=="pet" then
			local _,customName,_,_,_,_,_,name = C_PetJournal.GetPetInfoByPetID(petID)
			if customName and customName:match(searchText) then
				return true
			elseif name and name:match(searchText) then
				return true
			end
		elseif idType=="species" then
			local name = C_PetJournal.GetPetInfoBySpeciesID(petID)
			if name and name:match(searchText) then
				return true
			end
		elseif idType=="leveling" and (L["Leveling"]):match(searchText) then
			return true
		end
	end
end

-- for use by separate tab panel
function panel:GetTeamSearchInfo()
	return searchText,searchTabs
end

-- export needs access to this doh
function panel:GetWorkingList()
	return workingList
end

--[[ List UI ]]

function panel:FillTeamButton(button,key)
	button.key = key
	local title = rematch:GetTeamTitle(key)
	button.Name:SetText(title)
	if type(key)=="number" then -- if this is a targeted team (npcID for key)
		button.Name:SetTextColor(1,1,1)
		local npcName = rematch:GetNameFromNpcID(key)
		if title~=npcName and not settings.HideTargetNames then
			button.Name:SetHeight(21)
			button.SubName:SetText(npcName)
			button.SubName:Show()
		else
			button.Name:SetHeight(36)
			button.SubName:Hide()
		end
	else -- named team
		button.Name:SetTextColor(1,.82,0) -- colored gold
		button.Name:SetHeight(36) -- never has a target
		button.SubName:Hide()
	end
	local xoffset = -4
	if saved[key].notes then
		button.Notes:Show()
		xoffset = xoffset - 22
	else
		button.Notes:Hide()
	end
	if rematch:HasPreferences(saved[key]) then
		button.Preferences:SetPoint("RIGHT",xoffset,-1)
		button.Preferences:Show()
		xoffset = xoffset - 22
	else
		button.Preferences:Hide()
	end
	button.Name:SetPoint("TOPRIGHT",xoffset-2,-6)
	for i=1,3 do
		local petID = saved[key][i][1]
		local icon = rematch:GetPetIcon(petID)
		if not icon then -- pet is not found, get its species instead
			petID = saved[key][i][5]
			icon = rematch:GetPetIcon(petID)
		end
		button.Pets[i].petID = petID
		button.Pets[i].Icon:SetTexture(icon)
		button.Pets[i].Icon:SetDesaturated(type(petID)=="number" and petID~=0)
	end
end

function panel:UpdateList()
	local numData = #workingList
	local scrollFrame = panel.List.ScrollFrame
	local offset = HybridScrollFrame_GetOffset(scrollFrame)
	local buttons = scrollFrame.buttons
	rematch:HideTooltip()
	for i=1,#buttons do
		local index = i + offset
		local button = buttons[i]
		if ( index <= numData) then
			local key = workingList[index]
			panel:FillTeamButton(button,key)
			if key==settings.loadedTeam then
				button:SetBackdropBorderColor(1,0.82,0)
			else
				button:SetBackdropBorderColor(0.33,0.33,0.33)
			end
			button:Show()
		else
			button:Hide()
		end
	end

	HybridScrollFrame_Update(scrollFrame,44*numData,44)
end

-- single click of a team now loads teams
function panel:TeamOnClick(button)
	local key = self.key
	if button=="RightButton" then
		rematch:SetMenuSubject(key)
		rematch:ShowMenu("TeamMenu","cursor")
	else
		rematch:LoadTeam(key)
		if self==panel.Top.Team then
			rematch:ShowTeam(key)
		end
		if type(key)=="number" then
			rematch.recentTarget = key
		end
		rematch:UpdateUI()
	end
end

-- desaturates the loaded team, hides pets and sets team name to given text ("Loading..." "No Team Loaded" etc)
function panel:NotifyTeamLoading(text)
	if panel==false then
		panel.LoadedTeam.InsetBack:SetDesaturated(true)
		panel.LoadedTeam.Team.Name:SetText(format("%s%s",rematch.hexGrey,text))
		panel.LoadedTeam.Team.Notes:Hide()
		for i=1,3 do
			panel.LoadedTeam.Team.Pets[i]:Hide()
		end
	end
end

function panel:SearchBoxOnTextChanged()
	searchText = nil
	local text = self:GetText()
	if text and text:len()>0 then
		if rematch:GetIDType(text)=="pet" then -- this is a search for a petID (BattlePet-0-0000etc)
			searchText = text
		else -- this is a normal search
			searchText = rematch:DesensitizeText(text)
		end
	end
	panel:Update()
	rematch.TeamTabs:Update()
end

-- use this to set a search (aside from the user typing in the searchbox) to handle searchbox bits
function panel:SetTeamSearch(text)
	local searchBox = panel.Top.SearchBox
	searchBox:ClearFocus()
	searchBox.Instructions:SetShown(not text)
	searchBox.Clear:SetShown(text and true)
	searchBox:SetText(text or "")
end

-- regardless whether in journal or standalone, go to a team tab
-- if key is given, go to the team
function rematch:ShowTeam(key)
	if settings.Minimized and key then
		return
	end
	-- if key given, switch to the tab that contains the team (if it's not already selected)
	if key and saved[key] then -- and settings.SelectedTab~=(saved[key].tab or 1) then
		rematch.TeamTabs:SelectTeamTab(saved[key].tab or 1)
	end
	-- team panel not visible, show it
	if not panel:IsVisible() then
		if rematch.Frame:IsVisible() then
			settings.Minimized = nil
			settings.ActivePanel = 2
			rematch.Frame:ConfigureFrame()
		elseif rematch.Journal:IsVisible() then
			settings.JournalPanel = 1
			rematch.Journal:ConfigureJournal()
		else
			return -- neither frame or journal on screen
		end
	end
	-- if key provided, scroll to team (which should be on the team panel workingList at this point)
	if key and saved[key] then
		local index
		for i=1,#workingList do
			if workingList[i]==key then
				index = i
			end
		end
		if index then
			rematch:ListScrollToIndex(panel.List.ScrollFrame,index)
			rematch:ListBling(panel.List.ScrollFrame,"key",key)
		elseif not panel.retryShowTeam then
			-- if index not found then we didn't scroll to team yet, try again once things settle
			panel.retryShowTeam = true
			C_Timer.After(0.1,function() rematch:ShowTeam(key) end)
			return
		end
	end
	panel.retryShowTeam = nil
end

-- this registers/unregisters for UPDATE_MOUSEOVER_UNIT depending on settings
-- if off is true, turn off regardless of its settings (entering combat)
function rematch:UpdateAutoLoadState(off)
	if not off and settings.AutoLoad and not settings.AutoLoadTargetOnly then
		rematch:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	else
		rematch:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	end
end

-- ordinarily done in PetClicks template; handles special handling of team list pet buttons
-- to display team menu instead of pet menu
function panel:PetButtonOnClick(button)
	if button=="RightButton" then
		rematch:SetMenuSubject(self:GetParent().key)
		rematch:ShowMenu("TeamMenu","cursor")
	elseif IsModifiedClick("CHATLINK") then
		if rematch:GetIDType(self.petID)=="pet" then
			ChatEdit_InsertLink(C_PetJournal.GetBattlePetLink(self.petID))
		end
	else
		rematch:LockPetCard(self,self.petID)
	end
end

function panel:Resize(width)
	panel:SetWidth(width)
	for _,button in ipairs(panel.List.ScrollFrame.buttons) do
		button:SetWidth(width-32)
	end
	panel.Top:SetWidth(width)
	panel.Top.Team:SetWidth(width-17)
end

function panel:TeamOnEnter()
	self.Backplate:SetTexture(0.25,0.5,0.75)
	if self.Name:IsTruncated() or (self.SubName:IsVisible() and self.SubName:IsTruncated()) then
		local teamName = rematch:GetTeamTitle(self.key)
		if type(self.key)=="number" then
			local npcName = rematch:GetNameFromNpcID(self.key)
			rematch.ShowTooltip(self,teamName,self.key~=npcName and npcName)
		else
			rematch.ShowTooltip(self,format("%s%s",rematch.hexGold,teamName))
		end
	else
		rematch:HideTooltip()
	end
end

function panel:TeamOnLeave()
	self.Backplate:SetTexture(0.15,0.15,0.15)
	rematch:HideTooltip()
end